@@ -198,7 +198,7 @@ def pascalCaseToTitle(name: str, addSpaces=True) -> str:
     replace = r'\1\2'
   name = re.sub(r'(\w)([A-Z])', replace, name)
   name = name.replace('_', ' ')
-  return name.title()
+  return name.title().strip()
 
 # Make a class for reference persistence
 _nameFmtType = t.Callable[[str], str]
@@ -553,4 +553,18 @@ def convertImages(globstr:str='*.*', ext='png', replace=True, folder: FilePath=N
     print(
       f'Conversion errors occurred in the following files:\n'
       + '\n'.join(errs)
-    )
\ No newline at end of file
+    )
+
+def hierarchicalUpdate(curDict: dict, other: dict, replaceLists=False):
+  """Dictionary update that allows nested keys to be updated without deleting the non-updated keys"""
+  if other is None:
+    return curDict
+  for k, v in other.items():
+    curVal = curDict.get(k, None)
+    if isinstance(curVal, dict) and isinstance(v, dict):
+      hierarchicalUpdate(curVal, v)
+    elif replaceLists and isinstance(curVal, list) and isinstance(v, list):
+      curVal.extend(v)
+    else:
+      curDict[k] = v
+  return curDict
\ No newline at end of file
