@@ -8,6 +8,7 @@ from warnings import warn
 from pyqtgraph.Qt import QtCore, QtWidgets
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
+from utilitys.widgets import PopupLineEditor
 
 from . import PrjParam
 from .procwrapper import NestedProcWrapper
@@ -122,6 +123,15 @@ class ParamEditor(QtWidgets.QDockWidget):
     self.fileType = fileType
     """Used under the hood to name saved states"""
 
+    self.savedStates: t.List[str] = []
+    """Records which states are available since the last change"""
+
+    self.watcher = QtCore.QFileSystemWatcher()
+    """Responsible for detecting changes in the saved states"""
+    if self.saveDir is not None:
+      self.watcher.addPath(str(self.saveDir))
+    self.watcher.directoryChanged.connect(lambda: self.updateSavedStates())
+
     # -----------
     # Construct parameter tree
     # -----------
@@ -139,6 +149,10 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     self._buildGui()
 
+    # Populate initial values
+    if self.saveDir:
+      self.updateSavedStates()
+
     SPAWNED_EDITORS.append(weakref.proxy(self))
 
   def _paramTreeChanged(self, *_args, **_kwargs):
@@ -171,7 +185,7 @@ class ParamEditor(QtWidgets.QDockWidget):
                  allowOverwriteDefault=allowOverwriteDefault)
     # self.applyChanges()
     self.lastAppliedName = saveName
-    self.sigParamStateCreated.emit(str(saveName))
+    # TODO: Decide whether states created outside saveDir should be emitted here
     return paramState
 
   def saveCurStateAsDefault(self):
@@ -190,18 +204,32 @@ class ParamEditor(QtWidgets.QDockWidget):
 
   def loadParamValues(self, stateName: t.Union[str, Path],
                       stateDict: dict=None,
+                      useDefaults=True,
                       candidateParams: t.List[Parameter]=None,
-                      applyChanges=True):
+                      applyChanges=True, ):
     """
-    Can restore a state created by `fns.paramValues` If extra keys were saved (other than just 'value'), `hasExtraKeys`
-    must be set to *True*
+    Can restore a state created by `fns.paramValues`
+
+    :param stateName: State name to load, see ParamEditor.saveParamValues
+    :param stateDict: Override the state name values by providing a dict instead to prevent file IO
+    :param useDefaults: Whether default values should be used for all unspecified state values. If not, only the
+      specified keys will be updated.
+    :param candidateParams: set of parameters these settings should apply to. If *None*, defaults to all parameters in
+      this editor
+    :param applyChanges: Whether to trigger `sigChangesApplied` after load.
+    :return: Loaded state
     """
     loadDict = self._parseStateDict(stateName, stateDict)
     # First check for extra keys, will be the case if 'children' is one of the keys. Can't do value-loading in that case,
     # must do state-loading instead
     if 'children' in loadDict:
       return self._loadParamState(stateName, stateDict)
-    
+
+    defaultFile = self.formatFileName('Default')
+    if useDefaults and defaultFile and defaultFile.exists():
+      base = fns.attemptFileLoad(defaultFile)
+      loadDict = fns.hierarchicalUpdate(base, loadDict, replaceLists=True)
+
     if candidateParams is None:
       candidateParams = fns.params_flattened(self.params)
       
@@ -218,7 +246,8 @@ class ParamEditor(QtWidgets.QDockWidget):
       if isinstance(vv, dict):
         # Successively traverse down child tree
         curCandidates = [p for p in candidateParams if checkParentChain(p, kk)]
-        self.loadParamValues('', vv, curCandidates, applyChanges=False)
+        # By this point defaults are already populated so no need to reset them otherwise infinite recursion will occur
+        self.loadParamValues('', vv, useDefaults=False, candidateParams=curCandidates, applyChanges=False)
       else:
         unhandled[kk] = vv
     with self.params.treeChangeBlocker():
@@ -233,7 +262,8 @@ class ParamEditor(QtWidgets.QDockWidget):
                            f'{matches}')
     if applyChanges:
       self.applyChanges()
-    self.lastAppliedName = stateName
+    self.lastAppliedName = Path(stateName).stem
+    self.stateSelector.setText(self.lastAppliedName)
     return fns.paramValues(self.params, includeDefaults=True)
 
 
@@ -260,7 +290,6 @@ class ParamEditor(QtWidgets.QDockWidget):
     if not filename.exists():
       return
     filename.unlink()
-    self.sigParamStateDeleted.emit(stateName)
 
   def registerProps(self, constParams: t.List[PrjParam], namePath:t.Sequence[str]=(),
                      asProperty=True, **extraOpts):
@@ -450,6 +479,22 @@ class ParamEditor(QtWidgets.QDockWidget):
     for proc in self.procToParamsMapping:
       menu.addAction(proc.name, lambda _p=proc: _p.run())
     return menu
+  
+  def updateSavedStates(self):
+    dirGlob = list(self.saveDir.glob(f'*.{self.fileType}'))
+    dirStates = [f.stem for f in dirGlob]
+    mod: QtCore.QStringListModel = self.stateSelector.model
+
+    newStates = set(dirStates) - set(self.savedStates)
+    delStates = set(self.savedStates) - set(newStates)
+
+    self.savedStates = dirStates
+    mod.setStringList(self.savedStates)
+    for state in newStates:
+      self.sigParamStateCreated.emit(state)
+    for state in delStates:
+      self.sigParamStateDeleted.emit(state)
+
 
   def addDirItemsToMenu(self, parentMenu: QtWidgets.QMenu, removeExistingChildren=True):
     """Helper function for populating menu from directory contents"""
@@ -499,17 +544,25 @@ class ParamEditor(QtWidgets.QDockWidget):
     self.saveAsBtn = QtWidgets.QPushButton('Save As...')
     self.applyBtn = QtWidgets.QPushButton('Apply')
 
+    self.stateSelector = PopupLineEditor(clearOnComplete=False)
+    """Shows current state or load a new one"""
+
     # -----------
     # Widget layout
     # -----------
     children = [
+      [QtWidgets.QLabel('Current state: '), self.stateSelector],
       [self.expandAllBtn, self.collapseAllBtn],
       self.tree,
       [self.saveAsBtn, self.applyBtn]
     ]
+
+    self.dockContentsWidget = utilitys.widgets.EasyWidget.buildWidget(children)
+
     if self.saveDir is None:
       self.saveAsBtn.hide()
-    self.dockContentsWidget = utilitys.widgets.EasyWidget.buildWidget(children)
+      self.dockContentsWidget.easyChild.children_[0].widget_.hide()
+
     self.centralLayout = self.dockContentsWidget.easyChild.layout_
     self.setWidget(self.dockContentsWidget)
 
